// Server-side global variables
require(`dotenv`).config({path:`./config/.env`})
const path = require("path")

// Database
require(`./config/db`)


// Express
const express = require(`express`)
const app = express()


app.use(require(`body-parser`).json())
app.use(require(`cors`)({credentials: true, origin: process.env.LOCAL_HOST}))


// Routers
app.use(require(`./routes/cars`))
app.use(require(`./routes/users`))


const appPath = path.join(__dirname,"..","client/build")
app.use(express.static(appPath))
app.get("*",function(req,res)
{
    res.sendFile(path.resolve(appPath, "index.html"))    
})

// Port
app.listen(process.env.SERVER_PORT, () => 
{
    console.log(`Connected to port ` + process.env.SERVER_PORT)
})


// Error 404
app.use((req, res, next) => {next(createError(404))})


// Handle errors
app.use(function (err, req, res, next)
{       
    
    if (!err.statusCode) 
    {
        err.statusCode = 500
    }
    
    // check that all required paramters are not empty in any route
    if (err instanceof ReferenceError)
    {
        err.statusCode = 400
        err.message = "Cannot reference a variable that has not been declared. This can be caused in run-time if the user did not input a parameter that is required by a router"
    }    

    // Server-side error details
    console.log("Error Details...",err.message)
   
    res.status(err.statusCode).send(err.message)
})